@@ -17,10 +17,13 @@
 #ifndef _FIREGL_PUBLIC_H_
 #define _FIREGL_PUBLIC_H_
 
+#ifndef ESX 
 #include <stdarg.h>
+#include "kcl_iommu.h"
+#endif /*ifndef ESX*/ 
+
 #include "kcl_pci.h"
 #include "kcl_io.h"
-#include "kcl_iommu.h"
 
 #define FIREGL_USWC_SUPPORT     1
 
@@ -30,6 +33,7 @@
 #define LITTLEENDIAN_CPU 1
 #endif
 
+#ifndef ESX 
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
 
 #define REMAP_PAGE_RANGE_FN remap_pfn_range
@@ -51,6 +55,19 @@
                         (vma)->vm_end - (vma)->vm_start, \
                         (vma)->vm_page_prot)
 
+/* Page size*/
+#ifndef PAGE_SIZE_4K
+#define PAGE_SIZE_4K (4UL*1024)
+#endif
+#ifndef PAGE_SIZE_2M
+#define PAGE_SIZE_2M (2UL*1024*1024)
+#endif
+#ifndef PAGE_SIZE_4M
+#define PAGE_SIZE_4M (4UL*1024*1024)
+#endif
+#ifndef PAGE_SIZE_1G
+#define PAGE_SIZE_1G (1UL*1024*1024*1024)
+#endif
 
 /* Page table macros */
 
@@ -75,27 +92,29 @@ do { \
 } while(0)
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
-
-#define PMD_OFFSET(pmd_p, pgd_p, pte_linear) \
+#define PUD_PRESENT(pud_p) \
 do { \
-    pud_t *pud_p = pud_offset(pgd_p, pte_linear); \
-    if (!pud_present(*(pud_p)))	\
+    if (!pud_present(*(pud_p))) \
     { \
         return PAGING_FAULT_SIGBUS_INT;   /* Something bad happened; generate SIGBUS */ \
         /* alternatively we could generate a NOPAGE_OOM "out of memory" */ \
     } \
-    pmd_p = pmd_offset(pud_p, pte_linear); \
 } while(0)
 
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11) */
+#define PUD_OFFSET(pud_p, pgd_p, pte_linear)  \
+do { \
+    pud_p = pud_offset(pgd_p, pte_linear); \
+} while(0)
+
+#define PUD_HUGE(pud) ((pud_val(pud) & _PAGE_PSE) != 0)
+
+#endif
 
 #define PMD_OFFSET(pmd_p, pgd_p, pte_linear)	\
 do { \
     pmd_p = pmd_offset(pgd_p, pte_linear); \
 } while(0)
 
-#endif
-
 #define PMD_PRESENT(pmd_p) \
 do { \
     if (!pmd_present(*(pmd_p)))	\
@@ -107,6 +126,7 @@ do { \
 
 #define PMD_HUGE(pmd) ((pmd_val(pmd) & _PAGE_PSE) != 0)
 
+
 #ifdef pte_offset_atomic
 #define PTE_OFFSET(pte, pmd_p, pte_linear) \
 do { \
@@ -176,38 +196,52 @@ do { \
 
 struct vm_area_struct;
 struct semaphore;
+struct rw_semaphore;
 struct drm_device;
 struct firegl_pcie_mem;
 
-
 // note: assigning uniqe types to originally non interchangeable types
-typedef struct { int uniqe3; } kcl_sigset_t;
 typedef struct { int uniqe7; } kcl_file_operations_t;
 
 typedef	int (*kcl_read_proc_t)(
     char* page, char** start, kcl_off_t off, int count, int* eof, void* data);
 
+typedef int (*kcl_write_proc_t)(
+    void* file, const char *buffer, unsigned long count, void *data);
+
 typedef struct {
     const char*             name;
-    kcl_read_proc_t        f;
-    kcl_file_operations_t* fops;
+    kcl_read_proc_t         rp;
+    kcl_write_proc_t        wp;
+    kcl_file_operations_t*  fops;
 } kcl_proc_list_t;
 
 extern kcl_proc_list_t KCL_PROC_FileList[];
+#endif /*ifndef ESX*/
 
 typedef struct {
     unsigned long           signature;
     int                     privdevcount; // number of privdev structs
+#ifndef ESX
     kcl_proc_list_t *      proclist;
+#endif
     const char *            name;
-	unsigned int	        major_version;
-	unsigned int	        minor_version;
-	unsigned int            patchlevel;
-	const char *	        date;
-    kcl_sigset_t *         psigmask;
+    unsigned int	        major_version;
+    unsigned int	        minor_version;
+    unsigned int            patchlevel;
+    const char *	        date;
     void *                  privglobal; // private global context
 } kcl_device_t;
 
+/* console mode info */
+typedef struct {
+    unsigned int mode_width;   /*the width of the framebuffer*/
+    unsigned int mode_height;  /*the height of the framebuffer*/
+    unsigned int depth;        /*the depth of the framebuffer*/
+    unsigned int pitch;        /*the pitch of the framebuffer*/
+    unsigned long fb_base;     /*the base address of the framebuffer*/
+} kcl_console_mode_info_t;
+
 /*****************************************************************************/
 
 /** KCL declarations */
@@ -240,27 +274,35 @@ typedef int             KCL_TYPE_Uid;       /** User identifier */
  * Since atomic variables are intended for using in the concurrent
  * environment, volatile is used
  */
+#ifndef ESX
 typedef struct { volatile unsigned int counter; } KCL_TYPE_Atomic;
+#endif
 
 /** OS independent constant definitions */
 
 typedef enum
 {
+    KCL_ERROR_TIMED_OUT,
     KCL_ERROR_DEVICE_RESOURCE_BUSY,
     KCL_ERROR_INVALID_ARGUMENT,
     KCL_ERROR_PERMISSION_DENIED,
     KCL_ERROR_INVALID_ADDRESS,
     KCL_ERROR_INPUT_OUTPUT,
+#ifndef ESX
     KCL_ERROR_INVALID_SLOT,
+#endif /*ifndef ESX*/
     KCL_ERROR_OUT_OF_MEMORY,
     KCL_ERROR_OPERATION_NOT_PERMITTED,
     KCL_ERROR_DEVICE_NOT_EXIST,
     KCL_ERROR_INTERRUPTED_SYSTEM_CALL,
     KCL_ERROR_SIGNAL_INTERRUPTED_SYSTEM_CALL,
+#ifndef ESX
     KCL_ERROR_CORRUPTED_SHARED_LIB,
+#endif /*ifndef ESX*/
     KCL_ERROR_NUM
 } KCL_ENUM_ErrorCode;
 
+#ifndef ESX
 typedef enum
 {
     KCL_PROCESS_STATE_READY_TO_RUN,
@@ -290,9 +332,11 @@ typedef struct {
 	unsigned long freehigh;		// Available high memory size 
 	unsigned int mem_unit;		// Memory unit size in bytes 
 }KCL_SYS_MEM_INFO;
+#endif /*ifndef ESX*/
 
+
+#ifndef ESX 
 /** KCL function declarations */
-
 extern void          ATI_API_CALL KCL_GlobalKernelScheduler(void);
 extern int           ATI_API_CALL KCL_GetSignalStatus(void);
 extern unsigned int  ATI_API_CALL KCL_CurrentProcessIsTerminating(void);
@@ -300,6 +344,7 @@ extern void          ATI_API_CALL KCL_SetCurrentProcessState(KCL_ENUM_ProcessSta
 extern const char*   ATI_API_CALL KCL_GetModuleParamString(void);
 extern KCL_TYPE_Pid  ATI_API_CALL KCL_GetPid(void);
 extern KCL_TYPE_Tgid ATI_API_CALL KCL_GetTgid(void);
+extern void *        ATI_API_CALL KCL_GetGroupLeader(void);
 extern KCL_TYPE_Uid  ATI_API_CALL KCL_GetEffectiveUid(void);
 extern void          ATI_API_CALL KCL_DelayInMicroSeconds(unsigned long usecs);
 extern void          ATI_API_CALL KCL_DelayUseTSC(unsigned long usecs);
@@ -342,19 +387,22 @@ extern int           ATI_API_CALL KCL_AtomicTestAndToggleBit(int nr, volatile vo
 extern int           ATI_API_CALL KCL_PosixSecurityCapCheck(KCL_ENUM_PosixSecurityCap cap);
 
 /*****************************************************************************/
-
-extern int ATI_API_CALL drm_name_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_bios_version(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_interrupt_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL drm_mem_info(char* buf, char** start, kcl_off_t offset, int len, int *eof, void *data);
-extern int ATI_API_CALL drm_mem_info1(char* buf, char** start, kcl_off_t offset, int len, int *eof, void *data);
-extern int ATI_API_CALL drm_vm_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL drm_clients_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_lock_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_ptm_info(char* buf, char** start, kcl_off_t offset, int len, int *eof, void *data);
+extern int ATI_API_CALL drm_name_info(char* buf, int request, void* data);
+extern int ATI_API_CALL firegl_bios_version(char* buf, int request, void* data);
+extern int ATI_API_CALL firegl_interrupt_info(char* buf, int request, void* data);
+extern int ATI_API_CALL drm_mem_info(char* buf, int request, void *data);
+extern int ATI_API_CALL drm_mem_info1(char* buf, int request, void *data);
+extern int ATI_API_CALL drm_vm_info(char* buf, int request, void* data);
+extern int ATI_API_CALL drm_clients_info(char* buf, int request, void* data);
+extern int ATI_API_CALL firegl_lock_info(char* buf, int request, void* data);
+extern int ATI_API_CALL firegl_ptm_info(char* buf, int request, void *data);
 #ifdef DEBUG
-extern int ATI_API_CALL drm_bq_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL drm_bq_info(char* buf, int request, void* data);
 #endif
+extern int ATI_API_CALL firegl_debug_proc_read(char* buf, int request, void* data);
+
+extern int ATI_API_CALL firegl_debug_proc_write(void* file, const char *buffer, unsigned long count, void *data);
+
 extern int ATI_API_CALL firegl_interrupt_open(void* data, KCL_IO_FILE_Handle file);
 extern int ATI_API_CALL firegl_interrupt_release(KCL_IO_FILE_Handle file);
 extern unsigned int ATI_API_CALL firegl_interrupt_read(
@@ -370,9 +418,6 @@ extern int ATI_API_CALL firegl_interrupt_write(
                                     kcl_size_t user_buf_size, 
                                     kcl_loff_t *user_file_pos);
 
-extern int ATI_API_CALL firegl_debug_proc_read(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_debug_proc_write(void* file, const char *buffer, unsigned long count, void *data); 
-
 /*****************************************************************************/
 
 extern int ATI_API_CALL firegl_private_init (kcl_device_t *);
@@ -409,12 +454,6 @@ extern const unsigned int* ATI_API_CALL mc_heap_get_page_idx_list(void *pointer_
 
 /*****************************************************************************/
 
-extern void ATI_API_CALL KCL_SIGNAL_ConfigureMask(kcl_sigset_t *pSigMask);
-extern int ATI_API_CALL firegl_sig_notifier(void *priv);
-extern int              KCL_SIGNAL_Notifier(void *priv);
-extern void ATI_API_CALL KCL_SIGNAL_BlockAll(int (*notifier)(void *priv), void *pPriv, kcl_sigset_t *pSigMask);
-extern void ATI_API_CALL KCL_SIGNAL_UnblockAll(void);
-
 extern unsigned long ATI_API_CALL kcl__cmpxchg(volatile void *ptr, unsigned long old,                      
                       unsigned long new, int size);
 
@@ -442,6 +481,20 @@ extern int ATI_API_CALL KCL_SEMAPHORE_DownInterruptible(struct semaphore* sem);
 
 extern void ATI_API_CALL KCL_SPINLOCK_STATIC_Grab(kcl_device_t *dev, int ndx);
 extern void ATI_API_CALL KCL_SPINLOCK_STATIC_Release(kcl_device_t *dev, int ndx);
+//rw semaphore for GPU reset
+extern void ATI_API_CALL KCL_RW_SEMAPHORE_DownWrite(struct rw_semaphore* sem);
+
+extern void ATI_API_CALL KCL_RW_SEMAPHORE_UpWrite(struct rw_semaphore* sem);
+
+extern void ATI_API_CALL KCL_RW_SEMAPHORE_DownRead(struct rw_semaphore* sem);
+
+extern void ATI_API_CALL KCL_RW_SEMAPHORE_UpRead(struct rw_semaphore* sem);
+
+extern void ATI_API_CALL KCL_RW_SEMAPHORE_Init(struct rw_semaphore* sem);
+
+extern kcl_size_t ATI_API_CALL KCL_RW_SEMAPHORE_GetObjSize(void);
+
+
 #ifdef VCE_SUPPORT
 #define __KE_MAX_SPINLOCKS 9
 #else
@@ -475,9 +528,11 @@ extern void ATI_API_CALL KCL_MEM_IncPageCount_Mapping(void* page);
 extern int ATI_API_CALL KCL_MEM_VerifyReadAccess(void* addr, kcl_size_t size);
 extern int ATI_API_CALL KCL_MEM_VerifyWriteAccess(void* addr, kcl_size_t size);
 extern unsigned long ATI_API_CALL KCL_GetPageTableByVirtAddr(unsigned long virtual_addr, unsigned long* page_addr);
+extern unsigned int ATI_API_CALL KCL_GetPageSizeByVirtAddr(unsigned long virtual_addr, unsigned int* page_size);
 extern int ATI_API_CALL KCL_LockUserPages(unsigned long vaddr, unsigned long* page_list, unsigned int page_cnt);
+extern int ATI_API_CALL KCL_LockReadOnlyUserPages(unsigned long vaddr, unsigned long* page_list, unsigned int page_cnt);
 extern void ATI_API_CALL KCL_UnlockUserPages(unsigned long* page_list, unsigned int page_cnt);
-extern int ATI_API_CALL KCL_TestAndClearPageDirtyFlag(unsigned long virtual_addr);
+extern int ATI_API_CALL KCL_TestAndClearPageDirtyFlag(unsigned long virtual_addr, unsigned int page_size);
 extern unsigned long ATI_API_CALL KCL_MEM_AllocLinearAddrInterval(KCL_IO_FILE_Handle  file, unsigned long addr, unsigned long len, unsigned long pgoff);
 extern int ATI_API_CALL KCL_MEM_ReleaseLinearAddrInterval(unsigned long addr, unsigned long len);
 extern void* ATI_API_CALL KCL_MEM_MapPageList(unsigned long *pagelist, unsigned int count);
@@ -487,6 +542,9 @@ extern void* ATI_API_CALL KCL_MEM_MapPageListWc(unsigned long *pagelist, unsigne
 extern void ATI_API_CALL KCL_MEM_Unmap(void* addr);
 extern unsigned long ATI_API_CALL KCL_GetInitKerPte(unsigned long address);
 
+//UEFI call
+extern void ATI_API_CALL KCL_Get_Console_Mode(kcl_console_mode_info_t *console_mode);
+extern int ATI_API_CALL KCL_EFI_IS_ENABLED(void);
 
 /*****************************************************************************/
 
@@ -547,8 +605,6 @@ extern int ATI_API_CALL KCL_MEM_VM_MapRegion(KCL_IO_FILE_Handle filp,
 
 extern int ATI_API_CALL firegl_pci_save_state(KCL_PCI_DevHandle pdev, struct drm_device* dev);
 extern int ATI_API_CALL firegl_pci_restore_state(KCL_PCI_DevHandle pdev, struct drm_device* dev);
-extern int ATI_API_CALL firegl_cail_powerdown(struct drm_device* dev, int state);
-extern int ATI_API_CALL firegl_cail_powerup(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_save_framebuffer(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_restore_framebuffer(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_save_onchip_ram(struct drm_device* dev);
@@ -561,7 +617,6 @@ extern void ATI_API_CALL firegl_pm_lock_highmem_gart(struct drm_device* dev, int
 /*****************************************************************************/
 
 extern int ATI_API_CALL KCL_PM_Is_SuspendToRam(int state);
-
 /*****************************************************************************/
 
 /* global constants */
@@ -612,7 +667,7 @@ extern unsigned long        KCL_SYSINFO_TimerTicksPerSecond;
 #endif
 
 #endif //FIREGL_USWC_SUPPORT
-
+#endif /*ifndef ESX*/
 
 /*****************************************************************************
 *                                                                            *
@@ -655,6 +710,7 @@ typedef unsigned int (*KAS_IhRoutine_t)(void* pIhContext);
 #define KAS_RETCODE_TIMEOUT         2
 #define KAS_RETCODE_SIGNAL          3
 
+#ifndef ESX 
 /** \brief Interface functions */
 extern unsigned int  ATI_API_CALL KAS_Initialize(KAS_Initialize_t* pinit);
 extern unsigned int  ATI_API_CALL KAS_Ih_Execute(KAS_IhRoutine_t ih_routine,
@@ -753,6 +809,7 @@ extern void*         ATI_API_CALL KAS_AtomicExchangePointer(
 extern unsigned long ATI_API_CALL KAS_GetTickCounter(void);
 extern unsigned long ATI_API_CALL KAS_GetTicksPerSecond(void);
 extern  long ATI_API_CALL KAS_ScheduleTimeout(long n_jiffies);
+extern unsigned long ATI_API_CALL KCL_MsecToJiffes(unsigned int ms);
 
 /******************************************************************************
 **
@@ -797,13 +854,17 @@ extern int ATI_API_CALL firegl_asyncio_fasync(int fd,
                                               KCL_IO_FILE_Handle filp,
                                               int mode);
 
-
 extern void *ATI_API_CALL KCL_lock_init(void);
 extern void ATI_API_CALL KCL_lock_deinit(void *plock);
 extern void ATI_API_CALL KCL_spin_lock(void *lock);
 extern void ATI_API_CALL KCL_spin_unlock(void *lock);
 extern void ATI_API_CALL KCL_get_random_bytes(void *buf, int nbytes);
 extern void* ATI_API_CALL KCL_get_pubdev(void);
+extern void  ATI_API_CALL KCL_fpu_begin(void);
+extern void  ATI_API_CALL KCL_fpu_end(void);
+extern void* ATI_API_CALL KCL_create_proc_dir(void *root_dir, const char *name, unsigned int access);
+extern void ATI_API_CALL KCL_remove_proc_dir_entry(void *root, const char *name);
+extern void* ATI_API_CALL KCL_create_proc_entry(void *root_dir, const char *name, unsigned int access_mode, kcl_file_operations_t* fops, void *read_fn, void* write_fn, void *private_data);
 
 //The length of uuid, standardized by the Open Software Foundation, is 16.
 //In kernel, hard coded in function generate_random_uuid(). 
@@ -815,6 +876,7 @@ extern void ATI_API_CALL adapter_chain_cleanup(void);
 extern void ATI_API_CALL cf_object_init(void);
 extern void ATI_API_CALL cf_object_cleanup(void);
 extern int ATI_API_CALL firegl_init_device_list(int num_of_devices);
+extern int ATI_API_CALL firegl_realloc_device_list(int num_of_devices);
 extern void ATI_API_CALL firegl_cleanup_device_heads(void);
 extern int ATI_API_CALL firegl_get_dev(void *pubdev, KCL_PCI_DevHandle pdev);
 extern void *ATI_API_CALL firegl_find_device(int minor);
@@ -829,7 +891,17 @@ extern int libip_iommu_invalid_pri_request( KCL_PCI_DevHandle pdev,
                                             unsigned long fault_addr,
                                             KCL_IOMMU_req_perm_t perm);
 
-extern int libip_iommu_vas_term( KCL_PCI_DevHandle pdev, int  pasid);
+extern int libip_iommu_invalidate_pasid_ctx( KCL_PCI_DevHandle pdev, int  pasid);
 
+//libip call back entry when suspend. return 0 if success . 
+extern int ATI_API_CALL libip_suspend(struct drm_device* dev, int state);
+
+//libip call back entry when resume. return 0 if success . 
+extern int ATI_API_CALL libip_resume(struct drm_device* dev);
+#endif /*ifndef ESX*/
+
+#ifdef ESX
+#include "kcl_esx.h"
+#endif
 
 #endif /* _FIREGL_PUBLIC_H_ */
