@@ -137,6 +137,7 @@ extern "C" {
 #define GLX_CONTEXT_PROFILE_MASK_ARB              0x9126
 #define GLX_CONTEXT_CORE_PROFILE_BIT_ARB          0x00000001
 #define GLX_CONTEXT_COMPATIBILITY_PROFILE_BIT_ARB 0x00000002
+#define GLXBadProfileARB                          13
 #endif
 
 #ifndef GLX_SGIS_multisample
@@ -917,6 +918,13 @@ extern unsigned int glXPresentDesktopAMD(unsigned int presentTexture, unsigned i
 #define GLX_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB 0x8256
 #endif
 
+#ifndef GLX_ARB_context_flush_control
+#define GLX_ARB_context_flush_control
+#define GLX_CONTEXT_RELEASE_BEHAVIOR_ARB            0x2097
+#define GLX_CONTEXT_RELEASE_BEHAVIOR_NONE_ARB       0x0000
+#define GLX_CONTEXT_RELEASE_BEHAVIOR_FLUSH_ARB      0x2098
+#endif
+
 #ifdef __cplusplus
 }
 #endif
